
/**
 * DES Cipher Virtual Device Driver - Header File
 *
 * Copyright (c) 2010 Tomasz Pieczerak <tphaster at gmail.com>
 */

#ifndef __DES_CIPHER_VD_H
#define __DES_CIPHER_VD_H

#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>

/* Defines */
#define DEVICE_NAME     "des_cipher_vd"
#define MAX_BUF_SIZE    (size_t)(PAGE_SIZE<<8)   /* 1 MiB */

/* Macros */
#define round8(x)       (size_t)((x&7) ? x+8-(x&7) : x)
                        /* round up to the nearest multiple of 8 */

/* Driver Modes */
#define NOOP    0   /* inactive - refuses all read/write operations */
#define REGKEY  1   /* register key mode - write only */
#define ENCRYPT 2   /* encryption mode   - write/read (order important) */
#define DECRYPT 3   /* decryption mode   - write/read (order important) */

/* I/O Control Commands */
#define DESCVD_IOC_MAGIC     'g'    /* a magic 8bit number for ioctl */

#define DESCVD_IOC_NOOP      _IO(DESCVD_IOC_MAGIC, NOOP)
#define DESCVD_IOC_REGKEY    _IO(DESCVD_IOC_MAGIC, REGKEY)
#define DESCVD_IOC_ENCRYPT   _IO(DESCVD_IOC_MAGIC, ENCRYPT)
#define DESCVD_IOC_DECRYPT   _IO(DESCVD_IOC_MAGIC, DECRYPT)

#endif

