
#include <stdio.h>
#include <string.h>
#include <error.h>
#include <errno.h>

#include "des_cipher_vd.h"

int main ( void )
{
    int fd;
    char key[]  = {'a','c','f','[','h','R','3','g'};
    char bkey[] = {'d','e','v',':','D','A','5','8'};

    char msg[]  = "This is my very secret msg, no one should see it...";
    char msg1[] = "There is no system but GNU and Linux is one of its kernels. -- St. IGNUcius (aka Richard Stallman)";

    char enc[round8(sizeof msg)+1];
    memset(enc, 0, sizeof enc);

    char enc1[round8(sizeof msg1)+1];
    memset(enc1, 0, sizeof enc1);

    printf("\n=== DES Cipher Virtual Device Test ===\n\n");

    /* open device */
    fd = open("/dev/des_cipher_vd", O_RDWR);

    if (fd == -1)
        error(-1, errno, "OPEN: error on open()\n");

    /* send secret key */
    if (ioctl(fd, DESCVD_IOC_REGKEY))
        error(-1, errno, "IOCTL: error on ioctl()\n");

    if (write(fd, key, 8) != 8)
        perror("REGKEY: error on write()\n");

    /* encrypt message */
    if (ioctl(fd, DESCVD_IOC_ENCRYPT))
        error(-1, errno, "IOCTL: error on ioctl()\n");

    if (write(fd, msg, sizeof msg) != sizeof msg)
        perror("ENCRYPT: error on write()\n");

    if (read(fd, enc, round8(sizeof msg)) != round8(sizeof msg))
        perror("ENCRYPT: error on read()\n");

    printf("Original  message: %s\n", msg);
    printf("Encrypted message: %s\n", enc);

    /* decrypt message */
    memset(msg, 0, sizeof msg);

    if (ioctl(fd, DESCVD_IOC_DECRYPT))
        error(-1, errno, "IOCTL: error on ioctl()");

    if (write(fd, enc, round8(sizeof msg)) != round8(sizeof msg))
        perror("DECRYPT: error on write()\n");

    if (read(fd, msg, round8(sizeof msg)) != round8(sizeof msg))
        perror("DECRYPT: error happens on read()");

    printf("Encrypted message: %s\n", enc);
    printf("Original  message: %s\n", msg);

    printf("\n(these pairs should be identical, "
           "if not something is wrong ...)\n\n");

    /* send BAD secret key */
    printf("=== BAD secret key test ===\n\n");

    if (ioctl(fd, DESCVD_IOC_REGKEY))
        error(-1, errno, "IOCTL: error on ioctl()\n");

    if (write(fd, bkey, 8) != 8)
        perror("REGKEY: error on write()\n");

    /* decrypt message */
    memset(msg, 0, sizeof msg);

    if (ioctl(fd, DESCVD_IOC_DECRYPT))
        error(-1, errno, "IOCTL: error on ioctl()\n");

    if (write(fd, enc, round8(sizeof msg)) != round8(sizeof msg))
        perror("DECRYPT: error on write()\n");

    if (read(fd, msg, round8(sizeof msg)) != round8(sizeof msg))
        perror("DECRYPT: error on read()\n");

    printf("Encrypted message: %s\n", enc);
    printf("Original  message: %s\n", msg);

    printf("\n(these pairs should NOT be identical, "
           "if not something is wrong ...)\n\n");

    printf("=== Again ... ===\n");

    /* send secret key */
    if (ioctl(fd, DESCVD_IOC_REGKEY))
        error(-1, errno, "IOCTL: error on ioctl()\n");

    if (write(fd, key, 8) != 8)
        perror("REGKEY: error on write()\n");

    /* encrypt message */
    if (ioctl(fd, DESCVD_IOC_ENCRYPT))
        error(-1, errno, "IOCTL: error on ioctl()\n");

    if (write(fd, msg1, sizeof msg1) != sizeof msg1)
        perror("ENCRYPT: error on write()\n");

    if (read(fd, enc1, round8(sizeof msg1)) != round8(sizeof msg1))
        perror("ENCRYPT: error on read()\n");

    printf("Original  message: %s\n", msg1);
    printf("Encrypted message: %s\n", enc1);

    /* decrypt message */
    memset(msg1, 0, sizeof msg1);

    if (ioctl(fd, DESCVD_IOC_DECRYPT))
        error(-1, errno, "IOCTL: error on ioctl()");

    if (write(fd, enc1, round8(sizeof msg1)) != round8(sizeof msg1))
        perror("DECRYPT: error on write()\n");

    if (read(fd, msg1, round8(sizeof msg1)) != round8(sizeof msg1))
        perror("DECRYPT: error happens on read()");

    printf("Encrypted message: %s\n", enc1);
    printf("Original  message: %s\n", msg1);

    printf("\n(these pairs should be identical, "
           "if not something is wrong ...)\n\n");

    /* send BAD secret key */
    printf("=== BAD secret key test ===\n\n");

    if (ioctl(fd, DESCVD_IOC_REGKEY))
        error(-1, errno, "IOCTL: error on ioctl()\n");

    if (write(fd, bkey, 8) != 8)
        perror("REGKEY: error on write()\n");

    /* decrypt message */
    memset(msg1, 0, sizeof msg1);

    if (ioctl(fd, DESCVD_IOC_DECRYPT))
        error(-1, errno, "IOCTL: error on ioctl()\n");

    if (write(fd, enc1, round8(sizeof msg1)) != round8(sizeof msg1))
        perror("DECRYPT: error on write()\n");

    if (read(fd, msg1, round8(sizeof msg1)) != round8(sizeof msg1))
        perror("DECRYPT: error on read()\n");

    printf("Encrypted message: %s\n", enc1);
    printf("Original  message: %s\n", msg1);

    printf("\n(these pairs should NOT be identical, "
           "if not something is wrong ...)\n\n");

    /* close device */
    close(fd);

    return 0;
}

